/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
/*===========================================================================
 * 
 * Project : Freyja
 * Author  : Terry 'Mongoose' Hendrix II
 * Website : http://icculus.org/freyja/
 * Email   : mongooseichiban@gmail.com
 * Object  : MeshControl
 * License : No use w/o permission, (C) 2000-2007 Mongoose
 * Comments: This class is an event control for backend mesh, and
 *           the related user interface signals.
 *
 * 
 *-- History ------------------------------------------------- 
 *
 * 2007.04.10:
 * Mongoose - Created, reformed from various events and such.
 ==========================================================================*/

#ifndef GUARD__FREYJA3D_MESHCONTROL_H_
#define GUARD__FREYJA3D_MESHCONTROL_H_

#include <hel/math.h>
#include <hel/Ray.h>
#include <mstl/stack.h>

#include "Control.h"
#include "RecentFiles.h"


namespace freyja3d {

class MeshControl : public Control
{
 public:

	/* FreyjaControl options */
	typedef enum {
		fNone                = 0,
		fLoadMaterialInSlot  = 1,
		fLoadTextureInSlot   = 2,
		fUsingARBFragments   = 4

	} options_t;


	////////////////////////////////////////////////////////////
	// Constructors
	////////////////////////////////////////////////////////////

	static MeshControl* GetInstance() 
	{ return mInstance ? mInstance : ( mInstance = new MeshControl() ); }
	/*------------------------------------------------------
	 * Pre  : 
	 * Post : "Singleton" Constructor for MeshControl.
	 *
	 ------------------------------------------------------*/

	~MeshControl();
	/*------------------------------------------------------
	 * Pre  : 
	 * Post : Deconstructor for MeshControl.
	 *
	 ------------------------------------------------------*/


	////////////////////////////////////////////////////////////
	// Public Accessors
	////////////////////////////////////////////////////////////

	byte *GenerateTextureCheckerBoard(byte bg[4], byte fg[4],
									  uint32 width, uint32 height,
									  uint32 runlen);
	/*------------------------------------------------------
	 * Pre  : 
	 * Post : 
	 *
	 ------------------------------------------------------*/

	int32 GetTextureId() { return mTextureId; }
	/*------------------------------------------------------
	 * Pre  : 
	 * Post : Get the current active texture slot.
	 *
	 ------------------------------------------------------*/

	bool LoadMaterial(const char *filename);
	/*------------------------------------------------------
	 * Pre  : 
	 * Post : Unserializes current material and its metadata. 
	 *
	 ------------------------------------------------------*/

	void RefreshInterface();
	/*------------------------------------------------------
	 * Pre  : 
	 * Post : Completely reset material interface to 
	 *        match backend values.
	 *        Useful for changing materials in the editor.
	 *
	 ------------------------------------------------------*/

	bool SaveMaterial(const char *filename);
	/*------------------------------------------------------
	 * Pre  : 
	 * Post : Serializes current material and its metadata.
	 *
	 ------------------------------------------------------*/

	void SetAmbient(uint32 i, vec_t value);
	/*------------------------------------------------------
	 * Pre  : 
	 * Post : Set ambient material value per component.
	 *
	 ------------------------------------------------------*/

	void SetDiffuse(uint32 i, vec_t value);
	/*------------------------------------------------------
	 * Pre  : 
	 * Post : Set diffuse material value per component.
	 *
	 ------------------------------------------------------*/

	void SetEmissive(uint32 i, vec_t value);
	/*------------------------------------------------------
	 * Pre  : 
	 * Post : Set emissive material value per component.
	 *
	 ------------------------------------------------------*/

	void SetSpecular(uint32 i, vec_t value);
	/*------------------------------------------------------
	 * Pre  : 
	 * Post : Set specular material value per component.
	 *
	 ------------------------------------------------------*/


	////////////////////////////////////////////////////////////
	// Public Mutators
	////////////////////////////////////////////////////////////

	void AttachMethodListeners();
	/*------------------------------------------------------
	 * Pre  : 
	 * Post : Hooks up MethodDelegates to the event system.
	 ------------------------------------------------------*/

	bool InitRecentFilesMenu();
	/*------------------------------------------------------
	 * Pre  : 
	 * Post : Generates recent files menu data model.
	 *
	 ------------------------------------------------------*/

	bool LoadTexture(const char *filename);
	/*------------------------------------------------------
	 * Pre  : 
	 * Post : Returns true if sucessful.
	 *
	 ------------------------------------------------------*/

	bool LoadTextureBuffer(byte *image, uint32 width, uint32 height, 
						   uint32 bpp, uint32 colortype);
	/*------------------------------------------------------
	 * Pre  : 
	 * Post : Returns true if sucessful.
	 *
	 ------------------------------------------------------*/


	////////////////////////////////////////////////////////////
	// Public Properities 
	////////////////////////////////////////////////////////////

	uint32 mSelected;
	uint32 GetSelected() { return mSelected; }
	void SetSelected(uint32 i) { mSelected = i; }
	/*------------------------------------------------------
	 * Pre  : 
	 * Post : Returns currently selected material index.
	 ------------------------------------------------------*/


	////////////////////////////////////////////////////////////
	// Public Events 
	////////////////////////////////////////////////////////////

	uint32 EvSelectId;
	void EvSelect(uint32 value);
	/*------------------------------------------------------
	 * Pre  : 
	 * Post : Event for selecting current mesh by index.
	 *
	 ------------------------------------------------------*/

	uint32 EvSetNameId;
	void EvSetName(char *text);
	/*------------------------------------------------------
	 * Pre  : 
	 * Post : Event to set 'human readable' mesh name.
	 *
	 ------------------------------------------------------*/

	void EvNewMesh();
	/*------------------------------------------------------
	 * Pre  : 
	 * Post : Handle event for create new mesh.
	 *
	 ------------------------------------------------------*/

	void EvOpenMesh();
	/*------------------------------------------------------
	 * Pre  : 
	 * Post : Handle event for unserialize mesh.
	 *        Opens file dialog for user input.
	 *
	 ------------------------------------------------------*/

	void EvSaveMesh();
	/*------------------------------------------------------
	 * Pre  : 
	 * Post : Handle event for serialize material.
	 *        Opens file dialog for user input.
	 *
	 ------------------------------------------------------*/

	void EvRecentFile(uint32 value) 
	{ LoadMesh( mRecent.GetFilename(value) ); }
	/*------------------------------------------------------
	 * Pre  : 
	 * Post : Load a file from the recently used menu.
	 *
	 ------------------------------------------------------*/

	void EvNop() { Print("MeshControl::Nop() Called."); }
	/*------------------------------------------------------
	 * Pre  : 
	 * Post : NOP.
	 *
	 ------------------------------------------------------*/


protected:

	MeshControl();
	/*------------------------------------------------------
	 * Pre  : 
	 * Post : Constructor for MeshControl.
	 *
	 ------------------------------------------------------*/


 private:

	typedef void (MeshControl::*MethodPtr)();
	typedef void (MeshControl::*MethodPtr1f)(vec_t);
	typedef void (MeshControl::*MethodPtr1s)(char *);
	typedef void (MeshControl::*MethodPtr1u)(uint32);

	////////////////////////////////////////////////////////////
	// Private Accessors
	////////////////////////////////////////////////////////////

	uint32 CreateListener(const char *name, MethodPtr ptr);
	/*------------------------------------------------------
	 * Pre  : 
	 * Post : Bind a MethodDelegate to an external event.
	 *        Returns event Id.
	 *
	 ------------------------------------------------------*/

	uint32 CreateListener1f(const char *name, MethodPtr1f ptr);
	/*------------------------------------------------------
	 * Pre  : 
	 * Post : Bind a MethodDelegate to an external event.
	 *        Returns event Id.
	 *
	 ------------------------------------------------------*/

	uint32 CreateListener1s(const char *name, MethodPtr1s ptr);
	/*------------------------------------------------------
	 * Pre  : 
	 * Post : Bind a MethodDelegate to an external event.
	 *        Returns event Id.
	 *
	 ------------------------------------------------------*/

	uint32 CreateListener1u(const char *name, MethodPtr1u ptr);
	/*------------------------------------------------------
	 * Pre  : 
	 * Post : Bind a MethodDelegate to an external event.
	 *        Returns event Id.
	 *
	 ------------------------------------------------------*/


	////////////////////////////////////////////////////////////
	// Private Mutators
	////////////////////////////////////////////////////////////

	void SetFlag(options_t flag, bool b);
	/*------------------------------------------------------
	 * Pre  : 
	 * Post : Pretty interface for setting bitflags.
	 *
	 ------------------------------------------------------*/

	static MeshControl *mInstance;      /* "Singleton" instance. */

	uint32 mFlags;                      /* Options for this class */

	int32 mTextureId;                   /* Currently active texture slot. */

	RecentFiles mRecent;                /* Recent material files lists. */
};


////////////////////////////////////////////////////////////
// Inline Private Mutators
////////////////////////////////////////////////////////////

inline
void MeshControl::SetFlag(options_t flag, bool b)
{
	if (b) 
	{
		mFlags |= flag;
	}
	else 
	{
		mFlags &= ~flag;
	}
}


inline
bool MeshControl::InitRecentFilesMenu()
{
	Print("Loading %s...", mRecent.GetResourceFilename());
	if ( !mRecent.LoadResource() )
	{
		Print("Failed to load '%s'.", mRecent.GetResourceFilename() );
		return false;
	}

	return true;
}



} // namespace freyja3d 

#endif // GUARD__FREYJA3D_MESHCONTROL_H_
